<h2 id="pre-upgrade-checks">Pre-Upgrade checks</h2>

These will eventually become Goss tests.  After they are written provide instruction here on how to execute them and
from where.

1. Make sure current kubernetes nodes are in a Ready state.
1. Make sure etcd has quorum.
1. Make sure nexus is healthy and can pull images.
1. Ensure keycloak is healthy.
1. Ensure cray cli is configured and functioning (for bss primarily).

<h2 id="upgrade-master-nodes">Terminology</h2>

Throughout the guide the terms "stable" and "upgrade" are used in the context of NCNs. The "stable" NCN is the master
node you will be running all of these commands from and therefore will not be affecting the power state of. Clearly
then the "upgrade" node is the node you will be next upgrading.

In this way when doing a rolling upgrade of the entire cluster at some point you will need to transfer the
responsibility of the "stable" NCN to another master. However, you do not need to do this before you are ready to
upgrade that node.

It is also possible to do this from a "remote" node (like the PIT during a normal install) however this is beyond the
scope of this documentation.

<h2 id="prerequisite-steps">Prerequisite Steps</h2>

These steps should be taken regardless of the type of NCN you will be upgrading.

1. Get a token to use for authenticated communication with the gateway. These should be exported on a "stable" master
   node (i.e., one that you will not be upgrading in this iteration).
   > **`NOTE`** `api-gw-service-nmn.local` is legacy, and will be replaced with api-gw-service.nmn.

   ```bash
   ncn# export TOKEN=$(curl -k -s -S -d grant_type=client_credentials \
      -d client_id=admin-client \
      -d client_secret=`kubectl get secrets admin-client-auth -o jsonpath='{.data.client-secret}' | base64 -d` \
      https://api-gw-service-nmn.local/keycloak/realms/shasta/protocol/openid-connect/token | jq -r '.access_token')

2. Export additional variables to hold all the information we need:

    ```text
    ncn# export CSM_RELEASE=csm-x.y.z
    ncn# export UPGRADE_NCN=ncn-m001
   
    ncn# export STABLE_NCN=$(hostname)
    ncn# export UPGRADE_XNAME=$(curl -s -k -H "Authorization: Bearer ${TOKEN}" "https://api-gw-service-nmn.local/apis/sls/v1/search/hardware?extra_properties.Role=Management" | \
         jq -r ".[] | select(.ExtraProperties.Aliases[] | contains(\"$UPGRADE_NCN\")) | .Xname")
    ncn# export UPGRADE_IP_NMN=$(dig +short $UPGRADE_NCN.nmn)
    ```

    Double check the values returned by the commands:

    ```text
    ncn# echo $STABLE_NCN
    ncn-m002
   
    ncn# echo $UPGRADE_NCN
    ncn-m001
   
    ncn# echo $UPGRADE_XNAME
    x3000c0s1b0n0
   
    ncn# echo $UPGRADE_IP_NMN
    10.252.1.4
    ```

    If there are any incorrect values, correct them before proceeding.

3. Backup any files that are specific to the node to be upgraded. For example, if this is ncn-m001, you should backup
   the site link configuration files:

    ```
    ncn-m001# cat /etc/sysconfig/network/ifcfg-lan0
    
    NAME='External Site-Link'
    
    # Select the NIC(s) for direct, external access.
    BRIDGE_PORTS='em1'
    
    # Set static IP (becomes "preferred" if dhcp is enabled)
    # NOTE: IPADDR's route will override DHCPs.
    BOOTPROTO='static'
    IPADDR='172.30.52.183/20'    # i.e. 10.100.10.1/24
    PREFIXLEN='20' # i.e. 24
    
    # DO NOT CHANGE THESE:
    ONBOOT='yes'
    STARTMODE='auto'
    BRIDGE='yes'
    BRIDGE_STP='no'
    ```

    ```
    ncn-m001# cat /etc/sysconfig/network/ifroute-lan0
    default 172.30.48.1 - -
    ```

4. Open a new terminal window to the stable NCN to watch the etcd cluster status, as well as the Kubernetes node
   listing.  This will be useful to watch the progress of the node being upgraded, so leave it up and running.

   > NOTE: if you have CAN enabled you are access nodes via that address.
   >> example:
   >>> ncn # ip -o -4 addr list vlan007|awk -F" |/" '{print $7}'

   ```bash
   ncn# watch 'etcdctl --cacert=/etc/kubernetes/pki/etcd/ca.crt \
        --cert=/etc/kubernetes/pki/etcd/ca.crt  \
        --key=/etc/kubernetes/pki/etcd/ca.key \
        --endpoints=localhost:2379 member list; echo ""; kubectl get nodes'
   ```

5. Proceed either of the next steps:
   - [Upgrade Master Node](../stage2/k8s-master-node-upgrade.md)
   - [Upgrade Worker Node](../stage2/k8s-worker-node-upgrade.md)
   - [Storage Node Prerequisite Steps](../stage4/storage-prerequisite-steps.md)
