# Master Specific Steps

These are steps specific to upgrading a master node.

1. Determine if the master node being upgraded is the "first master" node.  This is the node others contact to join the
   Kubernetes cluster, and if this is the node being upgraded, we'll need to re-assign this role to another master
   before proceeding.  Run the following command:

    > **`NOTE`** If the node returned ***IS NOT*** the one being rebuilt, proceed to step 3 *(see below example)*.

    ```bash
    ncn# curl -s -k -H "Authorization: Bearer ${TOKEN}" https://api-gw-service-nmn.local/apis/bss/boot/v1/bootparameters?name=Global | \
         jq '.[] | ."cloud-init"."meta-data"."first-master-hostname"'
    "ncn-m002"
    ```

2. **If necessary**, reconfigure BSS to point to a different master node than we will be upgrading.

    1. In this case, we'll change from ncn-m001 to ncn-m002. You can do the reverse when rebuilding ncn-m001.

        ```bash
        ncn# csi handoff bss-update-cloud-init --set meta-data.first-master-hostname=$STABLE_NCN --limit Global
        ```

    2. Copy the `promote-initial-master.sh` from the `./scripts/k8s` directory in this repo to the new first master and execute
       it. This will put in place the files necessary for other members to join this cluster.

        ```bash
        ncn# scp promote-initial-master.sh $STABLE_NCN:/tmp
        ncn# ssh $STABLE_NCN /tmp/promote-initial-master.sh
       
        CONTROL_PLANE_HOSTNAME has been set to 10.252.1.2
        CONTROL_PLANE_ENDPOINT has been set to 10.252.1.2:6442
        Setting K8S_NODE_IP to 10.252.1.6 for KUBELET_EXTRA_ARGS and kubeadm config
        FIRST_MASTER_HOSTNAME has been set to ncn-m002
        IMAGE_REGISTRY has been set to docker.io
        ```

        **Note** the `FIRST_MASTER_HOSTNAME` in the output above might not match the node you desire. This is not an
       error, this master hostname is not the one that BSS uses and therefore is fine.

3. Stop the etcd service on the master node being upgraded.

   ```bash
   ncn# ssh $UPGRADE_NCN systemctl stop etcd.service
   ```

4. Get the member ID of the node to be upgraded:

    ```bash
    ncn# export MEMBER_ID=$(etcdctl --cacert=/etc/kubernetes/pki/etcd/ca.crt \
         --cert=/etc/kubernetes/pki/etcd/ca.crt \
         --key=/etc/kubernetes/pki/etcd/ca.key \
         --endpoints=localhost:2379 member list | \
         grep $UPGRADE_NCN | cut -d ',' -f1)
    ```

5. Using the member ID, remove the etcd member for the node being upgraded:

   ```bash
   ncn# etcdctl --cacert=/etc/kubernetes/pki/etcd/ca.crt \
        --cert=/etc/kubernetes/pki/etcd/ca.crt \
        --key=/etc/kubernetes/pki/etcd/ca.key \
        --endpoints=localhost:2379 member remove $MEMBER_ID
   ```

6. Prepare the etcd cluster to receive a request to join from the NCN to be upgraded:

    ```bash
     ncn# etcdctl --cacert=/etc/kubernetes/pki/etcd/ca.crt \
          --cert=/etc/kubernetes/pki/etcd/ca.crt  \
          --key=/etc/kubernetes/pki/etcd/ca.key \
          --endpoints=localhost:2379 \
          member add $UPGRADE_NCN --peer-urls=https://$UPGRADE_IP_NMN:2380
    ```

7. Use the script provided in this repository (in the `./scripts/k8s` directory)  to cordon/drain the node.  This will evacuate pods running on the node.

   ```bash
   ncn# remove-k8s-node.sh $UPGRADE_NCN
   ```

Proceed to [Common Upgrade Steps](../common/upgrade-steps.md)
